use std::sync::Arc;

use glam::Mat4;
use vulkano::{
    Validated,
    buffer::{AllocateBufferError, Subbuffer},
    memory::allocator::StandardMemoryAllocator,
};

use super::{mvp::MVP, transform::Transform};

#[derive(Default)]
pub struct Camera {
    view: Mat4,
    projection: Mat4,

    transform: Transform,
}

impl Camera {
    pub fn new(view: Mat4, projection: Mat4) -> Self {
        Self {
            view,
            projection,
            transform: Transform::default(),
        }
    }

    pub fn get_transform(&self) -> &Transform {
        &self.transform
    }

    pub fn get_transform_mut(&mut self) -> &mut Transform {
        &mut self.transform
    }

    pub fn set_projection(&mut self, projection: Mat4) {
        self.projection = projection;
    }

    pub fn create_buffer(
        &self,
        memory_allocator: &Arc<StandardMemoryAllocator>,
    ) -> Result<Subbuffer<[MVP]>, Validated<AllocateBufferError>> {
        MVP::new(&self.transform.get_mat4(), &self.view, &self.projection)
            .into_buffer(memory_allocator)
    }
}
